// AUTOGENERATED - DO NOT EDIT

package tdlib

import (
	"encoding/json"
	"fmt"
)

// MessageForwardOrigin Contains information about the origin of a forwarded message
type MessageForwardOrigin interface {
	GetMessageForwardOriginEnum() MessageForwardOriginEnum
}

// MessageForwardOriginEnum Alias for abstract MessageForwardOrigin 'Sub-Classes', used as constant-enum here
type MessageForwardOriginEnum string

// MessageForwardOrigin enums
const (
	MessageForwardOriginUserType       MessageForwardOriginEnum = "messageForwardOriginUser"
	MessageForwardOriginChatType       MessageForwardOriginEnum = "messageForwardOriginChat"
	MessageForwardOriginHiddenUserType MessageForwardOriginEnum = "messageForwardOriginHiddenUser"
	MessageForwardOriginChannelType    MessageForwardOriginEnum = "messageForwardOriginChannel"
)

func unmarshalMessageForwardOrigin(rawMsg *json.RawMessage) (MessageForwardOrigin, error) {

	if rawMsg == nil {
		return nil, nil
	}
	var objMap map[string]interface{}
	err := json.Unmarshal(*rawMsg, &objMap)
	if err != nil {
		return nil, err
	}

	switch MessageForwardOriginEnum(objMap["@type"].(string)) {
	case MessageForwardOriginUserType:
		var messageForwardOriginUser MessageForwardOriginUser
		err := json.Unmarshal(*rawMsg, &messageForwardOriginUser)
		return &messageForwardOriginUser, err

	case MessageForwardOriginChatType:
		var messageForwardOriginChat MessageForwardOriginChat
		err := json.Unmarshal(*rawMsg, &messageForwardOriginChat)
		return &messageForwardOriginChat, err

	case MessageForwardOriginHiddenUserType:
		var messageForwardOriginHiddenUser MessageForwardOriginHiddenUser
		err := json.Unmarshal(*rawMsg, &messageForwardOriginHiddenUser)
		return &messageForwardOriginHiddenUser, err

	case MessageForwardOriginChannelType:
		var messageForwardOriginChannel MessageForwardOriginChannel
		err := json.Unmarshal(*rawMsg, &messageForwardOriginChannel)
		return &messageForwardOriginChannel, err

	default:
		return nil, fmt.Errorf("Error UnMarshaling, unknown type:" + objMap["@type"].(string))
	}
}

// MessageForwardOriginUser The message was originally sent by a known user
type MessageForwardOriginUser struct {
	tdCommon
	SenderUserID int32 `json:"sender_user_id"` // Identifier of the user that originally sent the message
}

// MessageType return the string telegram-type of MessageForwardOriginUser
func (messageForwardOriginUser *MessageForwardOriginUser) MessageType() string {
	return "messageForwardOriginUser"
}

// NewMessageForwardOriginUser creates a new MessageForwardOriginUser
//
// @param senderUserID Identifier of the user that originally sent the message
func NewMessageForwardOriginUser(senderUserID int32) *MessageForwardOriginUser {
	messageForwardOriginUserTemp := MessageForwardOriginUser{
		tdCommon:     tdCommon{Type: "messageForwardOriginUser"},
		SenderUserID: senderUserID,
	}

	return &messageForwardOriginUserTemp
}

// GetMessageForwardOriginEnum return the enum type of this object
func (messageForwardOriginUser *MessageForwardOriginUser) GetMessageForwardOriginEnum() MessageForwardOriginEnum {
	return MessageForwardOriginUserType
}

// MessageForwardOriginChat The message was originally sent by an anonymous chat administrator on behalf of the chat
type MessageForwardOriginChat struct {
	tdCommon
	SenderChatID    int64  `json:"sender_chat_id"`   // Identifier of the chat that originally sent the message
	AuthorSignature string `json:"author_signature"` // Original message author signature
}

// MessageType return the string telegram-type of MessageForwardOriginChat
func (messageForwardOriginChat *MessageForwardOriginChat) MessageType() string {
	return "messageForwardOriginChat"
}

// NewMessageForwardOriginChat creates a new MessageForwardOriginChat
//
// @param senderChatID Identifier of the chat that originally sent the message
// @param authorSignature Original message author signature
func NewMessageForwardOriginChat(senderChatID int64, authorSignature string) *MessageForwardOriginChat {
	messageForwardOriginChatTemp := MessageForwardOriginChat{
		tdCommon:        tdCommon{Type: "messageForwardOriginChat"},
		SenderChatID:    senderChatID,
		AuthorSignature: authorSignature,
	}

	return &messageForwardOriginChatTemp
}

// GetMessageForwardOriginEnum return the enum type of this object
func (messageForwardOriginChat *MessageForwardOriginChat) GetMessageForwardOriginEnum() MessageForwardOriginEnum {
	return MessageForwardOriginChatType
}

// MessageForwardOriginHiddenUser The message was originally sent by a user, which is hidden by their privacy settings
type MessageForwardOriginHiddenUser struct {
	tdCommon
	SenderName string `json:"sender_name"` // Name of the sender
}

// MessageType return the string telegram-type of MessageForwardOriginHiddenUser
func (messageForwardOriginHiddenUser *MessageForwardOriginHiddenUser) MessageType() string {
	return "messageForwardOriginHiddenUser"
}

// NewMessageForwardOriginHiddenUser creates a new MessageForwardOriginHiddenUser
//
// @param senderName Name of the sender
func NewMessageForwardOriginHiddenUser(senderName string) *MessageForwardOriginHiddenUser {
	messageForwardOriginHiddenUserTemp := MessageForwardOriginHiddenUser{
		tdCommon:   tdCommon{Type: "messageForwardOriginHiddenUser"},
		SenderName: senderName,
	}

	return &messageForwardOriginHiddenUserTemp
}

// GetMessageForwardOriginEnum return the enum type of this object
func (messageForwardOriginHiddenUser *MessageForwardOriginHiddenUser) GetMessageForwardOriginEnum() MessageForwardOriginEnum {
	return MessageForwardOriginHiddenUserType
}

// MessageForwardOriginChannel The message was originally a post in a channel
type MessageForwardOriginChannel struct {
	tdCommon
	ChatID          int64  `json:"chat_id"`          // Identifier of the chat from which the message was originally forwarded
	MessageID       int64  `json:"message_id"`       // Message identifier of the original message
	AuthorSignature string `json:"author_signature"` // Original post author signature
}

// MessageType return the string telegram-type of MessageForwardOriginChannel
func (messageForwardOriginChannel *MessageForwardOriginChannel) MessageType() string {
	return "messageForwardOriginChannel"
}

// NewMessageForwardOriginChannel creates a new MessageForwardOriginChannel
//
// @param chatID Identifier of the chat from which the message was originally forwarded
// @param messageID Message identifier of the original message
// @param authorSignature Original post author signature
func NewMessageForwardOriginChannel(chatID int64, messageID int64, authorSignature string) *MessageForwardOriginChannel {
	messageForwardOriginChannelTemp := MessageForwardOriginChannel{
		tdCommon:        tdCommon{Type: "messageForwardOriginChannel"},
		ChatID:          chatID,
		MessageID:       messageID,
		AuthorSignature: authorSignature,
	}

	return &messageForwardOriginChannelTemp
}

// GetMessageForwardOriginEnum return the enum type of this object
func (messageForwardOriginChannel *MessageForwardOriginChannel) GetMessageForwardOriginEnum() MessageForwardOriginEnum {
	return MessageForwardOriginChannelType
}
